==> Morpho-0.2.tar.gz: Morpho-0.2/src/jp/morpho/config/NewNode.java <==
/*
 * Copyright (c) 2003, Morpho Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the Morpho Project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.morpho.config;

import java.lang.reflect.Constructor;

import org.xml.sax.Attributes;

/**
 * <P>
 * NewNode 饹ϡnew ɽޤ
 * new ˤϡarg get set put call  new ޤ뤳ȤǤޤ
 * </P>
 * <P>
 * 㤨 new ϡΤ褦һ˵Ҥޤ
 * </P>
 * <P>
 * <PRE>
 *     &lt;new class="jp.foo.SomeClass"&gt;
 *         &lt;arg&gt;someValue&lt;/arg&gt;
 *         &lt;set name="someMethod"&gt;someValue2&lt;/set&gt;
 *     &lt;/new&gt;
 * </PRE>
 * </P>
 * <P>
 * ϡΤ褦ʥɤ񤯤ȤƱǤ
 * </P>
 * <P>
 * <PRE>
 *     Object obj = new jp.foo.SomeClass("someValue");
 *     obj.setSomeMethod("someValue2");
 * </PRE>
 * </P>
 * @author Kumiko Hiroi
 */
public class NewNode extends Node
{

	/**
	 * <P>
	 * ƥΡɤȥΡɤ°ꤷ NewNode ֥Ȥޤ
	 * </P>
	 * @param parent ΥΡɤοƥΡ
	 * @param attributes ΥΡɤ°
	 */
	public NewNode(Node parent, Attributes attributes)
	{
		super(parent, attributes);
	}

	/**
	 * <P>
	 * ֤ͤޤ
	 * </P>
	 * @return 
	 * @see jp.morpho.config.Node#getTag()
	 */
	public int getTag()
	{
		return NodeTypes.NEW;
	}

	/**
	 * <P>
	 * ꤵ줿֥Ȥޤ
	 * 饹ˤ褦ˡ֥Ȥޤ
	 * </P>
	 * @param obj ֥Ȥκ˥ѥ᡼ȤƻѤ Node ֥
	 * @return 줿֥
	 * @exception ConfigureException ֥ȤǤʤä
	 * @see jp.morpho.config.Node#configure(Object)
	 */
	public Object configure(Object obj) throws ConfigureException
	{
		try
		{
			Class objCls = getNodeClass();
			int argLength = 0;
			int nextIndex = size();
			for (int i = 0; i < size(); i++)
			{
				Object o = get(i);
				if (o instanceof String)
				{
					continue;
				}
				if (((Node) o).getTag() != NodeTypes.ARG)
				{
					nextIndex = i;
					break;
				}
				argLength++;
			}

			Object[] arg = new Object[argLength];
			for (int i = 0, j = 0; j < argLength; i++)
			{
				Object o = get(i);
				if (o instanceof String)
				{
					continue;
				}
				arg[j++] = ((Node) o).getValue(obj);
			}

			Constructor[] constructors = objCls.getConstructors();
			for (int c = 0; constructors != null && c < constructors.length; c++)
			{
				if (constructors[c].getParameterTypes().length != argLength)
				{
					continue;
				}

				Object newObj = null;
				boolean succeeded = false;
				try
				{
					newObj = constructors[c].newInstance(arg);
					succeeded = true;
				}
				catch (IllegalAccessException e)
				{
				}
				catch (InstantiationException e)
				{
				}
				catch (IllegalArgumentException e)
				{
				}

				if (succeeded)
				{
					configure(newObj, nextIndex);
					return newObj;
				}
			}

			throw new IllegalStateException("No Constructor: " + getAttribute("class") + " on " + obj);
		}
		catch (Exception e)
		{
			throw new ConfigureException(e);
		}
	}

}

==> Morpho-1.0.tar.gz: Morpho-1.0/src/jp/morpho/config/NewNode.java <==
/*
 * Copyright (c) 2003, Morpho Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the Morpho Project nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package jp.morpho.config;

import java.lang.reflect.Constructor;

import org.xml.sax.Attributes;

/**
 * <P>
 * NewNode NX́Anew ^O\܂B
 * new ^Oɂ́Aarg Aget Aset Aput Acall  new ^O܂߂邱Ƃł܂B
 * </P>
 * <P>
 * Ⴆ new ^ÓÂ悤ɔzLqqɋLq܂B
 * </P>
 * <P>
 * <PRE>
 *     &lt;new class="jp.foo.SomeClass"&gt;
 *         &lt;arg&gt;someValue&lt;/arg&gt;
 *         &lt;set name="someMethod"&gt;someValue2&lt;/set&gt;
 *     &lt;/new&gt;
 * </PRE>
 * </P>
 * <P>
 * ́Â悤ȃR[hƂƓłB
 * </P>
 * <P>
 * <PRE>
 *     Object obj = new jp.foo.SomeClass("someValue");
 *     obj.setSomeMethod("someValue2");
 * </PRE>
 * </P>
 * @author Kumiko Hiroi
 */
public class NewNode extends Node
{

	/**
	 * <P>
	 * em[hƃm[h̑w肵 NewNode IuWFNg쐬܂B
	 * </P>
	 * @param parent ̃m[h̐em[h
	 * @param attributes ̃m[h̑
	 */
	public NewNode(Node parent, Attributes attributes)
	{
		super(parent, attributes);
	}

	/**
	 * <P>
	 * ^OlԂ܂B
	 * </P>
	 * @return ^Ol
	 * @see jp.morpho.config.Node#getTag()
	 */
	public int getTag()
	{
		return NodeTypes.NEW;
	}

	/**
	 * <P>
	 * w肳ꂽIuWFNg`܂B
	 * NX̐ɂ悤ɁAIuWFNg쐬܂B
	 * </P>
	 * @param obj IuWFNg̍쐬Ƀp[^ƂĎgp Node IuWFNg
	 * @return `ꂽIuWFNg
	 * @exception ConfigureException IuWFNg`łȂꍇ
	 * @see jp.morpho.config.Node#configure(Object)
	 */
	public Object configure(Object obj) throws ConfigureException
	{
		try
		{
			Class objCls = getNodeClass();
			int argLength = 0;
			int nextIndex = size();
			for (int i = 0; i < size(); i++)
			{
				Object o = get(i);
				if (o instanceof String)
				{
					continue;
				}
				if (((Node) o).getTag() != NodeTypes.ARG)
				{
					nextIndex = i;
					break;
				}
				argLength++;
			}

			Object[] arg = new Object[argLength];
			for (int i = 0, j = 0; j < argLength; i++)
			{
				Object o = get(i);
				if (o instanceof String)
				{
					continue;
				}
				arg[j++] = ((Node) o).getValue(obj);
			}

			Constructor[] constructors = objCls.getConstructors();
			for (int c = 0; constructors != null && c < constructors.length; c++)
			{
				if (constructors[c].getParameterTypes().length != argLength)
				{
					continue;
				}

				Object newObj = null;
				boolean succeeded = false;
				try
				{
					newObj = constructors[c].newInstance(arg);
					succeeded = true;
				}
				catch (IllegalAccessException e)
				{
				}
				catch (InstantiationException e)
				{
				}
				catch (IllegalArgumentException e)
				{
				}

				if (succeeded)
				{
					configure(newObj, nextIndex);
					return newObj;
				}
			}

			throw new IllegalStateException("No Constructor: " + getAttribute("class") + " on " + obj);
		}
		catch (Exception e)
		{
			throw new ConfigureException(e);
		}
	}

}
